package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1OutputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERGeneralizedTime;
import cn.com.infosec.asn1.DERIA5String;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.isismtt.ISISMTTObjectIdentifiers;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.CRLDistPoint;
import cn.com.infosec.asn1.x509.DistributionPoint;
import cn.com.infosec.asn1.x509.DistributionPointName;
import cn.com.infosec.asn1.x509.GeneralName;
import cn.com.infosec.asn1.x509.GeneralNames;
import cn.com.infosec.asn1.x509.PolicyInformation;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.jce.X509LDAPCertStoreParameters;
import cn.com.infosec.jce.exception.ExtCertPathValidatorException;
import cn.com.infosec.util.StoreException;
import cn.com.infosec.x509.ExtendedPKIXBuilderParameters;
import cn.com.infosec.x509.ExtendedPKIXParameters;
import cn.com.infosec.x509.X509AttributeCertStoreSelector;
import cn.com.infosec.x509.X509AttributeCertificate;
import cn.com.infosec.x509.X509CRLStoreSelector;
import cn.com.infosec.x509.X509CertStoreSelector;
import cn.com.infosec.x509.X509Store;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.cert.CertPath;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertStore;
import java.security.cert.CertStoreException;
import java.security.cert.Certificate;
import java.security.cert.PKIXParameters;
import java.security.cert.PolicyQualifierInfo;
import java.security.cert.TrustAnchor;
import java.security.cert.X509CRL;
import java.security.cert.X509CRLSelector;
import java.security.cert.X509CertSelector;
import java.security.cert.X509Certificate;
import java.security.cert.X509Extension;
import java.security.interfaces.DSAParams;
import java.security.interfaces.DSAPublicKey;
import java.security.spec.DSAPublicKeySpec;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.security.auth.x500.X500Principal;

public class CertPathValidatorUtilities {
   protected static final String CERTIFICATE_POLICIES;
   protected static final String BASIC_CONSTRAINTS;
   protected static final String POLICY_MAPPINGS;
   protected static final String SUBJECT_ALTERNATIVE_NAME;
   protected static final String NAME_CONSTRAINTS;
   protected static final String KEY_USAGE;
   protected static final String INHIBIT_ANY_POLICY;
   protected static final String ISSUING_DISTRIBUTION_POINT;
   protected static final String DELTA_CRL_INDICATOR;
   protected static final String POLICY_CONSTRAINTS;
   protected static final String FRESHEST_CRL;
   protected static final String CRL_DISTRIBUTION_POINTS;
   protected static final String AUTHORITY_KEY_IDENTIFIER;
   protected static final String ANY_POLICY = "2.5.29.32.0";
   protected static final String CRL_NUMBER;
   protected static final int KEY_CERT_SIGN = 5;
   protected static final int CRL_SIGN = 6;
   protected static final String[] crlReasons;

   static {
      CERTIFICATE_POLICIES = X509Extensions.CertificatePolicies.getId();
      BASIC_CONSTRAINTS = X509Extensions.BasicConstraints.getId();
      POLICY_MAPPINGS = X509Extensions.PolicyMappings.getId();
      SUBJECT_ALTERNATIVE_NAME = X509Extensions.SubjectAlternativeName.getId();
      NAME_CONSTRAINTS = X509Extensions.NameConstraints.getId();
      KEY_USAGE = X509Extensions.KeyUsage.getId();
      INHIBIT_ANY_POLICY = X509Extensions.InhibitAnyPolicy.getId();
      ISSUING_DISTRIBUTION_POINT = X509Extensions.IssuingDistributionPoint.getId();
      DELTA_CRL_INDICATOR = X509Extensions.DeltaCRLIndicator.getId();
      POLICY_CONSTRAINTS = X509Extensions.PolicyConstraints.getId();
      FRESHEST_CRL = X509Extensions.FreshestCRL.getId();
      CRL_DISTRIBUTION_POINTS = X509Extensions.CRLDistributionPoints.getId();
      AUTHORITY_KEY_IDENTIFIER = X509Extensions.AuthorityKeyIdentifier.getId();
      CRL_NUMBER = X509Extensions.CRLNumber.getId();
      crlReasons = new String[]{"unspecified", "keyCompromise", "cACompromise", "affiliationChanged", "superseded", "cessationOfOperation", "certificateHold", "unknown", "removeFromCRL", "privilegeWithdrawn", "aACompromise"};
   }

   protected static TrustAnchor findTrustAnchor(X509Certificate cert, Set trustAnchors) {
      Iterator trustAnchors1 = trustAnchors.iterator();
      TrustAnchor var2 = null;
      PublicKey var3 = null;
      Exception var4 = null;
      X509CertSelector var5;
      X509CertSelector var10000 = var5 = new X509CertSelector;
      X509Certificate var10001 = cert;
      var5.<init>();

      try {
         var10000.setSubject(getEncodedIssuerPrincipal(var10001).getEncoded());
      } catch (IOException var7) {
         throw new AnnotatedException("Cannot set subject search criteria for trust anchor.", var7);
      }

      while(trustAnchors1.hasNext() && var2 == null) {
         label62: {
            if ((var2 = (TrustAnchor)trustAnchors1.next()).getTrustedCert() != null) {
               if (var5.match(var2.getTrustedCert())) {
                  var3 = var2.getTrustedCert().getPublicKey();
                  break label62;
               }
            } else if (var2.getCAName() != null && var2.getCAPublicKey() != null) {
               label70: {
                  boolean var11;
                  boolean var13;
                  try {
                     var11 = getEncodedIssuerPrincipal(cert).equals(new X500Principal(var2.getCAName()));
                  } catch (IllegalArgumentException var9) {
                     var13 = false;
                     break label70;
                  }

                  if (var11) {
                     label71: {
                        PublicKey var12;
                        try {
                           var12 = var2.getCAPublicKey();
                        } catch (IllegalArgumentException var8) {
                           var13 = false;
                           break label71;
                        }

                        var3 = var12;
                        break label62;
                     }
                  }
               }
            }

            var2 = null;
         }

         if (var3 != null) {
            try {
               cert.verify(var3);
            } catch (Exception var6) {
               var4 = var6;
               var2 = null;
            }
         }
      }

      if (var2 == null && var4 != null) {
         throw new AnnotatedException("TrustAnchor found but certificate validation failed.", var4);
      } else {
         return var2;
      }
   }

   protected static void addAdditionalStoresFromAltNames(X509Certificate cert, ExtendedPKIXParameters pkixParams) {
      if (cert.getIssuerAlternativeNames() != null) {
         Iterator cert1 = cert.getIssuerAlternativeNames().iterator();

         while(cert1.hasNext()) {
            List var2;
            if ((var2 = (List)cert1.next()).get(0).equals(new Integer(6))) {
               addAdditionalStoreFromLocation((String)var2.get(1), pkixParams);
            }
         }
      }

   }

   protected static X500Principal getEncodedIssuerPrincipal(Object cert) {
      return cert instanceof X509Certificate ? ((X509Certificate)cert).getIssuerX500Principal() : (X500Principal)((X509AttributeCertificate)cert).getIssuer().getPrincipals()[0];
   }

   protected static Date getValidDate(PKIXParameters paramsPKIX) {
      Date paramsPKIX1;
      if ((paramsPKIX1 = paramsPKIX.getDate()) == null) {
         paramsPKIX1 = new Date.<init>();
      }

      return paramsPKIX1;
   }

   protected static X500Principal getSubjectPrincipal(X509Certificate cert) {
      return cert.getSubjectX500Principal();
   }

   protected static boolean isSelfIssued(X509Certificate cert) {
      return cert.getSubjectDN().equals(cert.getIssuerDN());
   }

   protected static DERObject getExtensionValue(X509Extension ext, String oid) {
      byte[] ext1;
      return (ext1 = ext.getExtensionValue(oid)) == null ? null : getObject(oid, ext1);
   }

   private static DERObject getObject(String param0, byte[] param1) {
      // $FF: Couldn't be decompiled
   }

   protected static X500Principal getIssuerPrincipal(X509CRL crl) {
      return crl.getIssuerX500Principal();
   }

   protected static AlgorithmIdentifier getAlgorithmIdentifier(PublicKey key) {
      try {
         return SubjectPublicKeyInfo.getInstance((new ASN1InputStream(key.getEncoded())).readObject()).getAlgorithmId();
      } catch (Exception var1) {
         throw new ExtCertPathValidatorException("Subject public key cannot be decoded.", var1);
      }
   }

   protected static final Collection findCRLs(X509CRLStoreSelector crlSelect, List crlStores) {
      List var10000 = crlStores;
      HashSet crlStores1;
      crlStores1 = new HashSet.<init>();
      Iterator var2 = var10000.iterator();
      AnnotatedException var3 = null;
      boolean var4 = false;

      while(true) {
         while(true) {
            if (!var2.hasNext()) {
               if (!var4 && var3 != null) {
                  throw var3;
               }

               return crlStores1;
            }

            Object var5;
            AnnotatedException var9;
            HashSet var10;
            if ((var5 = var2.next()) instanceof X509Store) {
               var10 = crlStores1;
               X509Store var11 = (X509Store)var5;

               try {
                  var10.addAll(var11.getMatches(crlSelect));
                  break;
               } catch (StoreException var6) {
                  var9 = new AnnotatedException.<init>("Exception searching in X.509 CRL store.", var6);
                  var3 = var9;
               }
            } else {
               var10 = crlStores1;
               CertStore var10001 = (CertStore)var5;

               try {
                  var10.addAll(var10001.getCRLs(crlSelect));
                  break;
               } catch (CertStoreException var7) {
                  var9 = new AnnotatedException.<init>("Exception searching in X.509 CRL store.", var7);
                  var3 = var9;
               }
            }
         }

         var4 = true;
      }
   }

   protected static final Set getQualifierSet(ASN1Sequence qualifiers) {
      HashSet var1;
      var1 = new HashSet.<init>();
      if (qualifiers == null) {
         return var1;
      } else {
         ASN1Sequence var10000 = qualifiers;
         ByteArrayOutputStream qualifiers1;
         qualifiers1 = new ByteArrayOutputStream.<init>();
         ASN1OutputStream var2;
         var2 = new ASN1OutputStream.<init>(qualifiers1);

         ByteArrayOutputStream var6;
         for(Enumeration var3 = var10000.getObjects(); var3.hasMoreElements(); var6.reset()) {
            try {
               var6 = qualifiers1;
               var2.writeObject(var3.nextElement());
               var1.add(new PolicyQualifierInfo(qualifiers1.toByteArray()));
            } catch (IOException var4) {
               throw new ExtCertPathValidatorException("Policy qualifier info cannot be decoded.", var4);
            }
         }

         return var1;
      }
   }

   protected static PKIXPolicyNode removePolicyNode(PKIXPolicyNode validPolicyTree, List[] policyNodes, PKIXPolicyNode _node) {
      PKIXPolicyNode var3 = (PKIXPolicyNode)_node.getParent();
      if (validPolicyTree == null) {
         return null;
      } else if (var3 != null) {
         var3.removeChild(_node);
         removePolicyNodeRecurse(policyNodes, _node);
         return validPolicyTree;
      } else {
         for(int validPolicyTree1 = 0; validPolicyTree1 < policyNodes.length; ++validPolicyTree1) {
            ArrayList _node1;
            _node1 = new ArrayList.<init>();
            policyNodes[validPolicyTree1] = _node1;
         }

         return null;
      }
   }

   private static void removePolicyNodeRecurse(List[] policyNodes, PKIXPolicyNode _node) {
      policyNodes[_node.getDepth()].remove(_node);
      if (_node.hasChildren()) {
         Iterator _node1 = _node.getChildren();

         while(_node1.hasNext()) {
            removePolicyNodeRecurse(policyNodes, (PKIXPolicyNode)_node1.next());
         }
      }

   }

   protected static boolean processCertD1i(int index, List[] policyNodes, DERObjectIdentifier pOid, Set pq) {
      List var4 = policyNodes[index - 1];

      for(int var5 = 0; var5 < var4.size(); ++var5) {
         PKIXPolicyNode var6;
         if ((var6 = (PKIXPolicyNode)var4.get(var5)).getExpectedPolicies().contains(pOid.getId())) {
            HashSet policyNodes1;
            HashSet var10003 = policyNodes1 = new HashSet;
            policyNodes1.<init>();
            var10003.add(pOid.getId());
            PKIXPolicyNode var9;
            PKIXPolicyNode var11 = var9 = new PKIXPolicyNode;
            ArrayList pOid1;
            pOid1 = new ArrayList.<init>();
            String var10 = pOid.getId();
            var11.<init>(pOid1, index, policyNodes1, var6, pq, var10, false);
            var6.addChild(var11);
            policyNodes[index].add(var9);
            return true;
         }
      }

      return false;
   }

   protected static void processCertD1ii(int index, List[] policyNodes, DERObjectIdentifier _poid, Set _pq) {
      List var4 = policyNodes[index - 1];

      for(int var5 = 0; var5 < var4.size(); ++var5) {
         PKIXPolicyNode var6;
         PKIXPolicyNode var10000 = var6 = (PKIXPolicyNode)var4.get(var5);
         var10000.getExpectedPolicies();
         if ("2.5.29.32.0".equals(var10000.getValidPolicy())) {
            HashSet policyNodes1;
            HashSet var10003 = policyNodes1 = new HashSet;
            policyNodes1.<init>();
            var10003.add(_poid.getId());
            PKIXPolicyNode var9;
            PKIXPolicyNode var12 = var9 = new PKIXPolicyNode;
            ArrayList _poid1;
            _poid1 = new ArrayList.<init>();
            String var11 = _poid.getId();
            var12.<init>(_poid1, index, policyNodes1, var6, _pq, var11, false);
            var6.addChild(var12);
            policyNodes[index].add(var9);
            return;
         }
      }

   }

   protected static void prepareNextCertB1(int i, List[] policyNodes, String id_p, Map m_idp, X509Certificate cert) {
      boolean var5 = false;
      Iterator var6 = policyNodes[i].iterator();

      while(var6.hasNext()) {
         PKIXPolicyNode var7;
         if ((var7 = (PKIXPolicyNode)var6.next()).getValidPolicy().equals(id_p)) {
            var5 = true;
            var7.expectedPolicies = (Set)m_idp.get(id_p);
            break;
         }
      }

      if (!var5) {
         Iterator var15 = policyNodes[i].iterator();

         while(var15.hasNext()) {
            PKIXPolicyNode var16;
            if ("2.5.29.32.0".equals((var16 = (PKIXPolicyNode)var15.next()).getValidPolicy())) {
               X509Certificate var10000 = cert;
               Set var17 = null;

               ASN1Sequence var21;
               try {
                  var21 = ASN1Sequence.getInstance(getExtensionValue(var10000, CERTIFICATE_POLICIES));
               } catch (Exception var11) {
                  throw new AnnotatedException("Certificate policies cannot be decoded.", var11);
               }

               Enumeration var19 = var21.getObjects();

               while(var19.hasMoreElements()) {
                  PolicyInformation var22;
                  try {
                     var22 = PolicyInformation.getInstance(var19.nextElement());
                  } catch (Exception var10) {
                     throw new AnnotatedException("Policy information cannot be decoded.", var10);
                  }

                  PolicyInformation var8 = var22;
                  if ("2.5.29.32.0".equals(var22.getPolicyIdentifier().getId())) {
                     Set var23;
                     try {
                        var23 = getQualifierSet(var8.getPolicyQualifiers());
                     } catch (CertPathValidatorException var9) {
                        throw new ExtCertPathValidatorException("Policy qualifier info set could not be built.", var9);
                     }

                     var17 = var23;
                     break;
                  }
               }

               boolean var20 = false;
               if (cert.getCriticalExtensionOIDs() != null) {
                  var20 = cert.getCriticalExtensionOIDs().contains(CERTIFICATE_POLICIES);
               }

               PKIXPolicyNode cert1;
               if ("2.5.29.32.0".equals((cert1 = (PKIXPolicyNode)var16.getParent()).getValidPolicy())) {
                  List[] var24 = policyNodes;
                  PKIXPolicyNode policyNodes1;
                  PKIXPolicyNode var10003 = policyNodes1 = new PKIXPolicyNode;
                  Map var10005 = m_idp;
                  ArrayList m_idp1;
                  m_idp1 = new ArrayList.<init>();
                  Set var18 = (Set)var10005.get(id_p);
                  var10003.<init>(m_idp1, i, var18, cert1, var17, id_p, var20);
                  cert1.addChild(var10003);
                  var24[i].add(policyNodes1);
               }
               break;
            }
         }
      }

   }

   protected static PKIXPolicyNode prepareNextCertB2(int i, List[] policyNodes, String id_p, PKIXPolicyNode validPolicyTree) {
      Iterator var4 = policyNodes[i].iterator();

      while(true) {
         PKIXPolicyNode var5;
         do {
            if (!var4.hasNext()) {
               return validPolicyTree;
            }
         } while(!(var5 = (PKIXPolicyNode)var4.next()).getValidPolicy().equals(id_p));

         ((PKIXPolicyNode)var5.getParent()).removeChild(var5);
         var4.remove();

         for(int var9 = i - 1; var9 >= 0; --var9) {
            List var6 = policyNodes[var9];

            PKIXPolicyNode var8;
            for(int var7 = 0; var7 < var6.size() && ((var8 = (PKIXPolicyNode)var6.get(var7)).hasChildren() || (validPolicyTree = removePolicyNode(validPolicyTree, policyNodes, var8)) != null); ++var7) {
            }
         }
      }
   }

   protected static boolean isAnyPolicy(Set policySet) {
      return policySet == null || policySet.contains("2.5.29.32.0") || policySet.isEmpty();
   }

   protected static void addAdditionalStoreFromLocation(String location, ExtendedPKIXParameters pkixParams) {
      if (pkixParams.isAdditionalLocationsEnabled()) {
         boolean var10000;
         boolean var10001;
         try {
            var10000 = location.startsWith("ldap://");
         } catch (Exception var14) {
            var10001 = false;
            throw new RuntimeException("Exception adding X.509 stores.");
         }

         if (var10000) {
            String var17;
            try {
               var17 = location.substring(7);
            } catch (Exception var13) {
               var10001 = false;
               throw new RuntimeException("Exception adding X.509 stores.");
            }

            location = var17;
            String var2 = null;

            int var18;
            try {
               var18 = var17.indexOf("/");
            } catch (Exception var12) {
               var10001 = false;
               throw new RuntimeException("Exception adding X.509 stores.");
            }

            StringBuffer var19;
            if (var18 != -1) {
               try {
                  var17 = location.substring(location.indexOf("/"));
               } catch (Exception var11) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }

               var2 = var17;

               try {
                  var19 = new StringBuffer;
               } catch (Exception var10) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }

               StringBuffer var3 = var19;

               String var10002;
               String var22;
               try {
                  var22 = location;
                  var10002 = location;
                  var3.<init>("ldap://");
               } catch (Exception var9) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }

               byte location1 = 0;

               try {
                  var17 = var19.append(var22.substring(location1, var10002.indexOf("/"))).toString();
               } catch (Exception var8) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }
            } else {
               try {
                  var19 = new StringBuffer;
               } catch (Exception var7) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }

               StringBuffer var23 = var19;

               try {
                  var23.<init>("ldap://");
                  var17 = var19.append(location).toString();
               } catch (Exception var6) {
                  var10001 = false;
                  throw new RuntimeException("Exception adding X.509 stores.");
               }
            }

            location = var17;

            ExtendedPKIXParameters var10003;
            ExtendedPKIXParameters var20;
            X509LDAPCertStoreParameters var10004;
            ExtendedPKIXParameters var21;
            ExtendedPKIXParameters var24;
            try {
               var21 = pkixParams;
               var24 = pkixParams;
               var20 = pkixParams;
               var10003 = pkixParams;
               var10004 = (new X509LDAPCertStoreParameters.Builder(location, var2)).build();
            } catch (Exception var5) {
               var10001 = false;
               throw new RuntimeException("Exception adding X.509 stores.");
            }

            X509LDAPCertStoreParameters location2 = var10004;

            try {
               var10003.addAddionalStore(X509Store.getInstance("CERTIFICATE/LDAP", location2, "INFOSEC"));
               var20.addAddionalStore(X509Store.getInstance("CRL/LDAP", location2, "INFOSEC"));
               var24.addAddionalStore(X509Store.getInstance("ATTRIBUTECERTIFICATE/LDAP", location2, "INFOSEC"));
               var21.addAddionalStore(X509Store.getInstance("CERTIFICATEPAIR/LDAP", location2, "INFOSEC"));
            } catch (Exception var4) {
               var10001 = false;
               throw new RuntimeException("Exception adding X.509 stores.");
            }
         }
      }
   }

   protected static Collection findCertificates(X509CertStoreSelector certSelect, List certStores) {
      List var10000 = certStores;
      HashSet certStores1;
      certStores1 = new HashSet.<init>();
      Iterator var2 = var10000.iterator();

      while(var2.hasNext()) {
         Object var3;
         HashSet var7;
         if ((var3 = var2.next()) instanceof X509Store) {
            var7 = certStores1;
            X509Store var10001 = (X509Store)var3;

            try {
               var7.addAll(var10001.getMatches(certSelect));
            } catch (StoreException var4) {
               throw new AnnotatedException("Problem while picking certificates from X.509 store.", var4);
            }
         } else {
            var7 = certStores1;
            CertStore var8 = (CertStore)var3;

            try {
               var7.addAll(var8.getCertificates(certSelect));
            } catch (CertStoreException var5) {
               throw new AnnotatedException("Problem while picking certificates from certificate store.", var5);
            }
         }
      }

      return certStores1;
   }

   protected static Collection findCertificates(X509AttributeCertStoreSelector certSelect, List certStores) {
      List var10000 = certStores;
      HashSet certStores1;
      certStores1 = new HashSet.<init>();
      Iterator var2 = var10000.iterator();

      while(var2.hasNext()) {
         Object var3;
         if ((var3 = var2.next()) instanceof X509Store) {
            HashSet var5 = certStores1;
            X509Store var10001 = (X509Store)var3;

            try {
               var5.addAll(var10001.getMatches(certSelect));
            } catch (StoreException var4) {
               throw new AnnotatedException("Problem while picking certificates from X.509 store.", var4);
            }
         }
      }

      return certStores1;
   }

   protected static void addAdditionalStoresFromCRLDistributionPoint(CRLDistPoint crldp, ExtendedPKIXParameters pkixParams) {
      if (crldp != null) {
         DistributionPoint[] var10000;
         try {
            var10000 = crldp.getDistributionPoints();
         } catch (Exception var5) {
            throw new AnnotatedException("Distribution points could not be read.", var5);
         }

         DistributionPoint[] crldp1 = var10000;

         for(int var2 = 0; var2 < crldp1.length; ++var2) {
            DistributionPointName var3;
            if ((var3 = crldp1[var2].getDistributionPoint()) != null && var3.getType() == 0) {
               GeneralName[] var7 = GeneralNames.getInstance(var3.getName()).getNames();

               for(int var4 = 0; var4 < var7.length; ++var4) {
                  if (var7[var4].getTagNo() == 6) {
                     addAdditionalStoreFromLocation(DERIA5String.getInstance(var7[var4].getName()).getString(), pkixParams);
                  }
               }
            }
         }
      }

   }

   protected static void getCRLIssuersFromDistributionPoint(DistributionPoint dp, Collection issuerPrincipals, X509CRLSelector selector, ExtendedPKIXParameters pkixParams) {
      ArrayList pkixParams1;
      pkixParams1 = new ArrayList.<init>();
      Iterator dp2;
      if (dp.getCRLIssuer() != null) {
         GeneralName[] var4 = dp.getCRLIssuer().getNames();

         for(int dp1 = 0; dp1 < var4.length; ++dp1) {
            if (var4[dp1].getTagNo() == 4) {
               try {
                  pkixParams1.add(new X500Principal(var4[dp1].getName().getDERObject().getEncoded()));
               } catch (IOException var6) {
                  throw new AnnotatedException("CRL issuer information from distribution point cannot be decoded.", var6);
               }
            }
         }
      } else {
         if (dp.getDistributionPoint() == null) {
            throw new AnnotatedException("CRL issuer is omitted from distribution point but no distributionPoint field present.");
         }

         dp2 = issuerPrincipals.iterator();

         while(dp2.hasNext()) {
            pkixParams1.add((X500Principal)dp2.next());
         }
      }

      dp2 = pkixParams1.iterator();

      while(dp2.hasNext()) {
         try {
            selector.addIssuerName(((X500Principal)dp2.next()).getEncoded());
         } catch (IOException var5) {
            throw new AnnotatedException("Cannot decode CRL issuer information.", var5);
         }
      }

   }

   private static BigInteger getSerialNumber(Object cert) {
      return cert instanceof X509Certificate ? ((X509Certificate)cert).getSerialNumber() : ((X509AttributeCertificate)cert).getSerialNumber();
   }

   protected static void getCertStatus(Date param0, X509CRL param1, Object param2, CertStatus param3) {
      // $FF: Couldn't be decompiled
   }

   protected static Set getDeltaCRLs(Date currentDate, ExtendedPKIXParameters paramsPKIX, X509CRL completeCRL) {
      X509CRLStoreSelector var3;
      var3 = new X509CRLStoreSelector.<init>();
      if (paramsPKIX.getDate() != null) {
         var3.setDateAndTime(paramsPKIX.getDate());
      } else {
         var3.setDateAndTime(currentDate);
      }

      try {
         var3.addIssuerName(getIssuerPrincipal(completeCRL).getEncoded());
      } catch (IOException var6) {
         new AnnotatedException("Cannot extract issuer from CRL.", var6);
      }

      X509CRL var10000 = completeCRL;
      BigInteger currentDate1 = null;

      boolean var10001;
      BigInteger var21;
      label73: {
         label72: {
            Exception var18;
            label77: {
               DERObject var19;
               try {
                  var19 = getExtensionValue(var10000, CRL_NUMBER);
               } catch (Exception var11) {
                  var18 = var11;
                  var10001 = false;
                  break label77;
               }

               DERObject var4 = var19;
               if (var19 == null) {
                  break label73;
               }

               try {
                  var21 = DERInteger.getInstance(var4).getPositiveValue();
                  break label72;
               } catch (Exception var10) {
                  var18 = var10;
                  var10001 = false;
               }
            }

            Exception currentDate2 = var18;
            throw new AnnotatedException("CRL number extension could not be extracted from CRL.", currentDate2);
         }

         currentDate1 = var21;
      }

      byte[] var20;
      try {
         var21 = currentDate1;
         var20 = completeCRL.getExtensionValue(ISSUING_DISTRIBUTION_POINT);
      } catch (Exception var5) {
         throw new AnnotatedException("Issuing distribution point extension value could not be read.", var5);
      }

      byte[] completeCRL1 = var20;
      BigInteger var17;
      if (var21 == null) {
         var17 = null;
      } else {
         var17 = currentDate1.add(BigInteger.valueOf(1L));
      }

      var3.setMinCRLNumber(var17);
      var3.setIssuingDistributionPoint(completeCRL1);
      var3.setIssuingDistributionPointEnabled(true);
      var3.setMaxBaseCRLNumber(currentDate1);
      HashSet var23 = new HashSet;
      HashSet currentDate3;
      HashSet var22 = currentDate3 = var23;
      X509CRLStoreSelector var10002 = var3;
      ExtendedPKIXParameters var10003 = paramsPKIX;
      HashSet var10004 = currentDate3;
      X509CRLStoreSelector var10005 = var3;
      ExtendedPKIXParameters var10006 = paramsPKIX;
      HashSet var10007 = currentDate3;
      X509CRLStoreSelector var10008 = var3;
      ExtendedPKIXParameters var10009 = paramsPKIX;
      currentDate3.<init>();

      AnnotatedException var24;
      label78: {
         try {
            var10007.addAll(findCRLs(var10008, var10009.getAdditionalStores()));
         } catch (AnnotatedException var9) {
            var24 = var9;
            var10001 = false;
            break label78;
         }

         try {
            var10004.addAll(findCRLs(var10005, var10006.getStores()));
         } catch (AnnotatedException var8) {
            var24 = var8;
            var10001 = false;
            break label78;
         }

         try {
            var22.addAll(findCRLs(var10002, var10003.getCertStores()));
            return var23;
         } catch (AnnotatedException var7) {
            var24 = var7;
            var10001 = false;
         }
      }

      AnnotatedException currentDate4 = var24;
      throw new AnnotatedException("Could not search for delta CRLs.", currentDate4);
   }

   protected static Set getCompleteCRLs(DistributionPoint dp, Object cert, Date currentDate, ExtendedPKIXParameters paramsPKIX) {
      Object var10000 = cert;
      X509CRLStoreSelector var4;
      var4 = new X509CRLStoreSelector.<init>();

      AnnotatedException dp1;
      AnnotatedException var16;
      boolean var10001;
      HashSet var18;
      label84: {
         label88: {
            try {
               var18 = new HashSet;
            } catch (AnnotatedException var13) {
               var16 = var13;
               var10001 = false;
               break label88;
            }

            HashSet var5 = var18;

            boolean var17;
            try {
               var18.<init>();
               var17 = var10000 instanceof X509AttributeCertificate;
            } catch (AnnotatedException var12) {
               var16 = var12;
               var10001 = false;
               break label88;
            }

            if (var17) {
               try {
                  var5.add(((X509AttributeCertificate)cert).getIssuer().getPrincipals()[0]);
               } catch (AnnotatedException var11) {
                  var16 = var11;
                  var10001 = false;
                  break label88;
               }
            } else {
               try {
                  var5.add(getEncodedIssuerPrincipal(cert));
               } catch (AnnotatedException var10) {
                  var16 = var10;
                  var10001 = false;
                  break label88;
               }
            }

            try {
               getCRLIssuersFromDistributionPoint(dp, var5, var4, paramsPKIX);
               break label84;
            } catch (AnnotatedException var9) {
               var16 = var9;
               var10001 = false;
            }
         }

         dp1 = var16;
         new AnnotatedException("Could not get issuer information from distribution point.", dp1);
      }

      if (cert instanceof X509Certificate) {
         var4.setCertificateChecking((X509Certificate)cert);
      } else if (cert instanceof X509AttributeCertificate) {
         var4.setAttrCertificateChecking((X509AttributeCertificate)cert);
      }

      if (paramsPKIX.getDate() != null) {
         var4.setDateAndTime(paramsPKIX.getDate());
      } else {
         var4.setDateAndTime(currentDate);
      }

      var4.setCompleteCRLEnabled(true);
      HashSet var19 = new HashSet;
      HashSet dp2;
      var18 = dp2 = var19;
      X509CRLStoreSelector var10002 = var4;
      ExtendedPKIXParameters var10003 = paramsPKIX;
      HashSet var10004 = dp2;
      X509CRLStoreSelector var10005 = var4;
      ExtendedPKIXParameters var10006 = paramsPKIX;
      HashSet var10007 = dp2;
      X509CRLStoreSelector var10008 = var4;
      ExtendedPKIXParameters var10009 = paramsPKIX;
      dp2.<init>();

      label65: {
         label89: {
            try {
               var10007.addAll(findCRLs(var10008, var10009.getStores()));
            } catch (AnnotatedException var8) {
               var16 = var8;
               var10001 = false;
               break label89;
            }

            try {
               var10004.addAll(findCRLs(var10005, var10006.getAdditionalStores()));
            } catch (AnnotatedException var7) {
               var16 = var7;
               var10001 = false;
               break label89;
            }

            try {
               var18.addAll(findCRLs(var10002, var10003.getCertStores()));
               break label65;
            } catch (AnnotatedException var6) {
               var16 = var6;
               var10001 = false;
            }
         }

         dp1 = var16;
         throw new AnnotatedException("Could not search for CRLs.", dp1);
      }

      if (!var19.isEmpty()) {
         return dp2;
      } else {
         throw new AnnotatedException("No CRLs found.");
      }
   }

   protected static Date getValidCertDateFromValidityModel(ExtendedPKIXParameters paramsPKIX, CertPath certPath, int index) {
      if (paramsPKIX.getValidityModel() == 1) {
         if (index <= 0) {
            return getValidDate(paramsPKIX);
         } else {
            int paramsPKIX1;
            if ((paramsPKIX1 = index - 1) == 0) {
               DERGeneralizedTime var10000;
               try {
                  var10000 = DERGeneralizedTime.getInstance(((X509Certificate)certPath.getCertificates().get(paramsPKIX1)).getExtensionValue(ISISMTTObjectIdentifiers.id_isismtt_at_dateOfCertGen.getId()));
               } catch (IllegalArgumentException var4) {
                  throw new AnnotatedException("Date of cert gen extension could not be read.");
               }

               DERGeneralizedTime index1 = var10000;
               if (var10000 != null) {
                  try {
                     return index1.getDate();
                  } catch (ParseException var3) {
                     throw new AnnotatedException("Date from dat of cert gen extension could not be parsed.", var3);
                  }
               } else {
                  return ((X509Certificate)certPath.getCertificates().get(paramsPKIX1)).getNotBefore();
               }
            } else {
               return ((X509Certificate)certPath.getCertificates().get(paramsPKIX1)).getNotBefore();
            }
         }
      } else {
         return getValidDate(paramsPKIX);
      }
   }

   protected static PublicKey getNextWorkingKey(List certs, int index) {
      PublicKey var2;
      if (!((var2 = ((Certificate)certs.get(index)).getPublicKey()) instanceof DSAPublicKey)) {
         return var2;
      } else {
         DSAPublicKey var9;
         if ((var9 = (DSAPublicKey)var2).getParams() != null) {
            return var9;
         } else {
            PublicKey var3;
            DSAPublicKey var11;
            do {
               ++index;
               if (index >= certs.size()) {
                  throw new CertPathValidatorException("DSA parameters cannot be inherited from previous certificate.");
               }

               if (!((var3 = ((X509Certificate)certs.get(index)).getPublicKey()) instanceof DSAPublicKey)) {
                  throw new CertPathValidatorException("DSA parameters cannot be inherited from previous certificate.");
               }
            } while((var11 = (DSAPublicKey)var3).getParams() == null);

            DSAParams certs1 = var11.getParams();
            DSAPublicKeySpec index1;
            DSAPublicKeySpec var10000 = index1 = new DSAPublicKeySpec;
            BigInteger certs2 = var9.getY();
            BigInteger var10 = certs1.getP();
            BigInteger var12 = certs1.getQ();
            BigInteger var4 = certs1.getG();
            var10000.<init>(certs2, var10, var12, var4);

            try {
               return KeyFactory.getInstance("DSA", "INFOSEC").generatePublic(index1);
            } catch (Exception var5) {
               throw new RuntimeException(var5.getMessage());
            }
         }
      }
   }

   protected static Collection findIssuerCerts(X509Certificate cert, ExtendedPKIXBuilderParameters pkixParams) {
      X509CertStoreSelector var2;
      X509CertStoreSelector var10000 = var2 = new X509CertStoreSelector;
      X509Certificate var10001 = cert;
      var2.<init>();
      HashSet cert1;
      cert1 = new HashSet.<init>();

      try {
         var10000.setSubject(var10001.getIssuerX500Principal().getEncoded());
      } catch (IOException var4) {
         throw new AnnotatedException("Subject criteria for certificate selector to find issuer certificate could not be set.", var4);
      }

      Iterator var17;
      label60: {
         AnnotatedException var13;
         label64: {
            ArrayList var14;
            boolean var15;
            try {
               var14 = new ArrayList;
            } catch (AnnotatedException var9) {
               var13 = var9;
               var15 = false;
               break label64;
            }

            ArrayList var3;
            ArrayList var16 = var3 = var14;

            X509CertStoreSelector var10002;
            ExtendedPKIXBuilderParameters var10003;
            ArrayList var10004;
            X509CertStoreSelector var10005;
            ExtendedPKIXBuilderParameters var10006;
            try {
               var10002 = var2;
               var10003 = pkixParams;
               var10004 = var3;
               var10005 = var2;
               var10006 = pkixParams;
               var3.<init>();
               var3.addAll(findCertificates(var2, pkixParams.getCertStores()));
            } catch (AnnotatedException var8) {
               var13 = var8;
               var15 = false;
               break label64;
            }

            try {
               var10004.addAll(findCertificates(var10005, var10006.getStores()));
            } catch (AnnotatedException var7) {
               var13 = var7;
               var15 = false;
               break label64;
            }

            try {
               var16.addAll(findCertificates(var10002, var10003.getAdditionalStores()));
            } catch (AnnotatedException var6) {
               var13 = var6;
               var15 = false;
               break label64;
            }

            try {
               var17 = var14.iterator();
               break label60;
            } catch (AnnotatedException var5) {
               var13 = var5;
               var15 = false;
            }
         }

         AnnotatedException cert2 = var13;
         throw new AnnotatedException("Issuer certificate cannot be searched.", cert2);
      }

      Iterator pkixParams1 = var17;

      while(pkixParams1.hasNext()) {
         cert1.add((X509Certificate)pkixParams1.next());
      }

      return cert1;
   }
}
